<?php

namespace Drupal\challenge_mitigation\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Provides a filter form for Challenge Mitigation IP whitelist.
 *
 * Allows administrators to filter whitelisted IPs by address.
 */
class ChallengeMitigationIpFilterForm extends FormBase {

  /**
   * Constructs a new ChallengeMitigationIpFilterForm.
   *
   * @param \Symfony\Component\HttpFoundation\RequestStack $requestStack
   *   The request stack service.
   */
  public function __construct(RequestStack $requestStack) {
    $this->requestStack = $requestStack;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): static {
    return new static(
      $container->get('request_stack')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'challenge_mitigation_ip_filter_form';
  }

  /**
   * {@inheritdoc}
   *
   * Builds the filter form for IP addresses.
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $request = $this->requestStack->getCurrentRequest();
    $ip_filter = $request->query->get('ip_filter') ?? '';

    $form['ip_filter'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Filter by IP address'),
      '#default_value' => $ip_filter,
      '#size' => 20,
      '#attributes' => [
        'placeholder' => $this->t('Enter IP address...'),
      ],
    ];

    $form['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Filter'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   *
   * Handles form submission and redirects to the whitelist page
   * with the entered filter as a query parameter.
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $value = $form_state->getValue('ip_filter');
    $query = ['ip_filter' => $value];
    $form_state->setRedirect('entity.cm_whitelist_ip.collection', [], ['query' => $query]);
  }

}
